#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a drawing look with an abstract effect

# (c) Dinasset 2021
#
#   History:
#
#   v1.0: 2021-01-31: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
from gimpfu import *
import random
import time
import datetime

# define here some constant value, when required
FixWH=1500

############################################################################
# service functions
#---------------------------------------------------------------------------
############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

############
# kris-color
# ---------- 
def kris_color_edge (inImage, inLayer, inColor, inSize):
    
    pdb.gimp_selection_all(inImage)
    pdb.gimp_selection_shrink(inImage, inSize)
    pdb.gimp_selection_invert(inImage)
    pdb.gimp_context_set_background(inColor)
    pdb.gimp_edit_fill(inLayer, FILL_BACKGROUND)
    pdb.gimp_selection_none(inImage)
    
    return (inLayer)

###########################
#   M   A   I   N
#--------------------------

def DrawStyleO (inImage, inDrawable, widthheight,
                doFrame, doFlatten) :
    
    pdb.gimp_context_push()
   
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    
    Name = inDrawable.name
    if '.' in Name:
        split = Name.split(".", 1)
        Name = split[0]
    Width = inDrawable.width
    Height = inDrawable.height
    Type = inImage.base_type
    Pos = pdb.gimp_image_get_item_position (inImage, inDrawable)
    
    # do your stuff ........................................................
    
    
    if widthheight == 0 :   # means the user wants to have output size same as input size
        widthheight = inImage.width+inImage.height      # real size of the input image
    
    # calculate scale factor for the new output image
    realWidthHeight = inImage.width+inImage.height      # real size of the input image
    if widthheight == 0 :   # means the user wants to have same size as input
        widthheight = realWidthHeight                                    
        scaleu = 1
    else :     
        optiWidthHeight = widthheight       # size to work on to get best results
        scaleu = (optiWidthHeight*1.0)/(realWidthHeight*1.0)      # ratio for scaling

    uwidth=int(Width*scaleu)    # user
    uheight=int(Height*scaleu)  # user
    
    scalew = (FixWH*1.0)/(realWidthHeight*1.0)  # work
    wwidth=int(Width*scalew)    # working
    wheight=int(Height*scalew)  # working
    
    newImage = pdb.gimp_image_duplicate (inImage)                    
    newBG = newImage.layers[0]     # BG layer
    # ------------------------------------------------------------------------
    # ------------------------------------------------------------------------
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "Background Image Layer"
    newBG.opacity = 100.0
    (Imean,sd,me,px,cn,pc) = pdb.gimp_drawable_histogram (newBG, HISTOGRAM_VALUE,
                                                          0.0, 1.0)
    #info_msg("input mean="+str(Imean))
    
    # take care of a possible existing selection
    NOsel = pdb.gimp_selection_is_empty(newImage)
    if NOsel == False:
        info_msg ("there is a selection")
        old_selection = pdb.gimp_selection_save (newImage)

    Original = newBG.copy()
    newImage.add_layer (Original, 1)
    pdb.gimp_item_set_visible (Original, False)
    Original.name = Name+" (original)"
    
    pdb.gimp_drawable_equalize (newBG, True)
    
    pdb.gimp_image_scale (newImage, wwidth, wheight)
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # Create new layers for the "abstract" effect
    Layer1 = newBG.copy()
    newImage.add_layer (Layer1, Pos)
    Layer1.opacity = 100.0
    Layer1.name = "Layer1: Base "
    Layer1.mode = LAYER_MODE_NORMAL 
    # blur it --------------------------------------------
    pdb.plug_in_gauss_iir2 (newImage, Layer1, 3, 3)
    # posterize ------------------------------------------
    #Posterized = Posterize (newImage, Layer1, 127)
    Poster = Layer1.copy()
    newImage.add_layer (Poster, Pos)
    pdb.gimp_drawable_posterize (Poster, 31)
    
    Poster.name = "Poster: Base posterized "
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # ----------------------------------------------------
    #[G'MIC] Color Abstraction Paint: fx_ColorAbstractionPaint 5,5,0.5,0,0.5,0,0.5,0,0,0,0,0,0,0
    pdb.plug_in_gmic_qt(newImage, Poster, 1,2,
        "-v - -fx_ColorAbstractionPaint 5,5,0.5,0,0.5,0,0.5,0,0,0,0,0,0,0 ")
    Abstract = newImage.layers[Pos]
    Abstract.opacity = 75.0
    Abstract.name = "Abstract: 1st.EffectLayer"
    Abstract.mode = LAYER_MODE_NORMAL
    #[G'MIC] Abstraction: fx_color_abstraction 0,100,0.01,0,50,50
    pdb.plug_in_gmic_qt(newImage, Poster, 1,2,
        "-v - -fx_color_abstraction 0,100,0.01,0,50,50 ")
    Abstract2 = newImage.layers[Pos]
    Abstract2.opacity = 75.0
    Abstract2.name = "Abstract2: 2nd.EffectLayer"
    Abstract2.mode = LAYER_MODE_GRAIN_MERGE
    pdb.plug_in_colortoalpha (newImage, Abstract2, (1.0,1.0,1.0,1.0))
    #
    Intermediate = pdb.gimp_layer_new_from_visible (newImage, newImage, "Intermediate")
    newImage.add_layer (Intermediate, Pos)
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # warning: G'MIC DOODLE does not handle correctly the selection !!!
    # hence we remove it temporarily
    if NOsel == False:
        pdb.gimp_selection_all (newImage)
    
    #[G'MIC] Doodle: fx_doodle 30,2,2,1.5,2,70,1
    pdb.plug_in_gmic_qt(newImage, Intermediate, 1,2,
        "-v - -fx_doodle 30,2,2,1.5,1,30,0 ")   # no preview
    Doodle = newImage.layers[Pos]
    Doodle.opacity = 75.0
    Doodle.name = "Doodle: 3rd.EffectLayer"
    Doodle.mode = LAYER_MODE_NORMAL
    pdb.plug_in_colortoalpha (newImage, Doodle, (1.0,1.0,1.0,1.0))
    pdb.plug_in_spread (newImage, Doodle, 1.0, 1.0)
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # let's keep the SAME selection for Plasma
    Plasma = pdb.gimp_layer_new (newImage, Width, Height, Type,
                                 "Plasma", 50.0, LAYER_MODE_LCH_CHROMA)
    newImage.add_layer (Plasma, Pos)
    pdb.plug_in_plasma (newImage, Plasma, 1, 0.25)
    
    #pdb.gimp_display_new (newImage)
    #return

    # now cut the NON SELECTED AREA
    if NOsel == False:
        pdb.gimp_image_select_item (newImage, CHANNEL_OP_REPLACE,
                                    old_selection)
        pdb.gimp_selection_invert (newImage)
        pdb.gimp_edit_cut (Doodle)
        pdb.gimp_layer_add_alpha (Plasma)
        pdb.gimp_edit_cut (Plasma)
        pdb.gimp_selection_invert (newImage)
    
    #pdb.gimp_display_new (newImage)
    #return

    #  
    
    if doFlatten == True:
        outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_IMAGE)
    else:    
        outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "visible") 
        newImage.add_layer (outcome, Pos)
    outcome.opacity = 100.0
    outcome.name = st+" "+Name+", with a sort of abstract look"
    (Omean,sd,me,px,cn,pc) = pdb.gimp_drawable_histogram (outcome, HISTOGRAM_VALUE,
                                                          0.0, 1.0)
    #7info_msg("output mean="+str(Omean))
    Mratio=Imean/Omean
    #info_msg("ratio of means="+str(Mratio))
    #info_msg("adjusting curves")
    pdb.gimp_drawable_curves_spline (outcome, HISTOGRAM_VALUE,
                                     6, (0.0,0.0,0.5,0.5*Mratio,1.0,1.0))
    
    # end do your stuff ......................................................
    
    if doFrame == True:
        # frame
        # - variables used
        (MeanR, B,C,D,E,F) = pdb.gimp_drawable_histogram (outcome, HISTOGRAM_RED, 0., 1.)
        (MeanG, B,C,D,E,F) = pdb.gimp_drawable_histogram (outcome, HISTOGRAM_GREEN, 0., 1.)
        (MeanB, B,C,D,E,F) = pdb.gimp_drawable_histogram (outcome, HISTOGRAM_BLUE, 0., 1.)
        if (MeanR+MeanG+MeanB)/3 < 100:
            mult = 2.5
        elif (MeanR+MeanG+MeanB)/3 < 150:
            mult = 1.6
        elif (MeanR+MeanG+MeanB)/3 < 200:
            mult = 1.2
        else:
            mult = 1.00
        frameColor = (min(255,int(MeanR*mult)),
                      min(255,int(MeanG*mult)),
                      min(255,int(MeanB*mult)),
                      255)                          # WARNING: int() is important !
        #gimp.message(str(frameColor))
        frameSize = int(round(FixWH/75))
        frameGranu = int(round(FixWH/200))
        rWidth = int(round(wwidth+frameSize*1.0))
        rHeight = int(round(wheight+frameSize*1.0))
        pdb.gimp_image_resize (newImage, rWidth, rHeight,
                               int(round(frameSize*0.5)), int(round(frameSize*0.5)))
        FrameLayer = pdb.gimp_layer_new(newImage, rWidth, rHeight, RGBA_IMAGE,
                        "White Border Layer", 100.0, LAYER_MODE_NORMAL)
        newImage.add_layer(FrameLayer, 0)
        pdb.gimp_edit_clear(FrameLayer)
        kris_color_edge(newImage, FrameLayer, frameColor, frameSize)
        pdb.gimp_layer_scale (FrameLayer, int(round(rWidth/frameGranu)),
                              int(round(rHeight/frameGranu)), True)
        pdb.plug_in_spread(newImage, FrameLayer, int(round(frameSize/frameGranu)),
                           int(round(frameSize/frameGranu)))
        kris_color_edge(newImage, FrameLayer, frameColor, 1)
        pdb.gimp_layer_scale(FrameLayer, rWidth, rHeight, True)
        pdb.gimp_image_select_item(newImage, CHANNEL_OP_REPLACE, FrameLayer)
        pdb.gimp_selection_invert(newImage)
        pdb.gimp_edit_clear(FrameLayer)
        pdb.gimp_selection_invert(newImage)
        pdb.gimp_edit_clear(FrameLayer)
        pdb.gimp_context_set_background(frameColor)
        pdb.gimp_edit_fill(FrameLayer, FILL_BACKGROUND)
        pdb.gimp_selection_none(newImage)
        kris_color_edge(newImage, FrameLayer, frameColor, 1)
        pdb.plug_in_gauss_rle(newImage, FrameLayer, frameSize, True, True)
        outcome = pdb.gimp_image_merge_down (newImage, FrameLayer, EXPAND_AS_NECESSARY)
    
    if widthheight<>FixWH:
        #info_msg ("final rescaling")
        pdb.gimp_image_scale (newImage, uwidth, uheight)   # to user desired size
        
    # end do your stuff .....
    
    pdb.gimp_display_new (newImage)
    
    pdb.gimp_context_pop()
    
    return(outcome)

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/Drawing/"
copyrightYear="2021"
desc="Draw Effect Style O (variant O2)"

register(
    "DrawStyleO",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+"...",
    "*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of Out Image"+
                        "\n (0=same as input)", 3000, (0, 9600, 200)),
      (PF_TOGGLE, "frame", "Add a frame?", True),
      (PF_TOGGLE, "flatten", "Do flatten?", True),
    ],
    [
      (PF_DRAWABLE, "outdrawable", "Output drawable", None),
    ],
    DrawStyleO,
    menu=menu
    )

main()
